@@ -3,16 +3,16 @@
 # see R/gdalalg.R and src/gdalalg.h
 # Chris Toney <jctoney at gmail.com>
 
-#' Convenience functions for using \dQuote{gdal} CLI algorithms
+#' Convenience functions for using GDAL CLI algorithms
 #'
 #' @name gdal_cli
 #' @description
-#' This set of functions can be used to access and run GDAL utilities as
-#' \dQuote{gdal} command line interface (CLI) algorithms.
+#' This set of functions can be used to access and run GDAL utilities as `gdal`
+#' command line interface (CLI) algorithms.
 #'
-#' **Requires GDAL >= `r .GDALALG_MIN_GDAL_STR`**
+#' **Requires GDAL >= 3.11.3**
 #'
-#' **Experimental** (see "Development status")
+#' **Experimental** (see the section `Development Status` below)
 #'
 #' @details
 #' These functions are convenient for accessing and running GDAL CLI algorithms
@@ -20,18 +20,18 @@
 #' documentation for additional information (`?GDALAlg`).
 #'
 #' `gdal_commands()` prints a list of commands and their descriptions to the
-#' console, and returns (invisibly) a data frame with columns named `command`,
-#' `description` and `URL`. The `contains` argument can be used to filter the
+#' console, and returns (invisibly) a data frame with columns `command`,
+#' `description`, `URL`. The `contains` argument can be used to filter the
 #' output, e.g., `gdal_commands("vector")` to return only commands for working
 #' with vector inputs.
 #'
 #' `gdal_usage()` prints a help message to the console for a given command, or
-#' for the root `"gdal"` entry point if called with no argument. No return
-#' value, called for that side effect only.
+#' for the root `gdal` entry point if called with no argument. No return value,
+#' called for that side effect only.
 #'
 #' `gdal_run()` executes a GDAL CLI algorithm and returns it as an object of
 #' class [`GDALAlg`][GDALAlg]. A list containing algorithm output(s) can be
-#' accessed by calling the \code{$outputs()} (plural) method on the returned
+#' accessed by calling the \code{$outputs()} method (plural) on the returned
 #' object, or, more conveniently in most cases, by calling \code{$output()}
 #' (singular) to return the the single output value when there is only one.
 #' After assigning the output, or otherwise completing work with the `GDALAlg`
@@ -49,25 +49,33 @@
 #' for class `GDALAlg`. Executing the returned algorithm is optional by calling
 #' the object's \code{$run()} method (assuming argument values were given).
 #'
+#' `gdal_global_reg_names()` returns a character vector containing the names of
+#' the algorithms in the GDAL global algorithm registry. These are the
+#' top-level nodes (`raster`, `vector`, etc.) known to GDAL. Potentially code
+#' external to GDAL could register a new command available for CLI use in a
+#' GDAL plugin. This function may be useful in certain troubleshooting
+#' scenarios. It will return a vector of length zero if no names are returned
+#' from the global registry.
+#'
 #' @param contains Optional character string for filtering output to certain
 #' commands, e.g., `gdal_commands("vector")`.
 #' @param recurse Logical value, `TRUE` to include all subcommands recursively
-#' (the default). Set to `FALSE` to include only the top-level \dQuote{gdal}
-#' commands (i.e., "raster", "vector", etc.)
+#' (the default). Set to `FALSE` to include only the top-level `gdal` commands
+#' (i.e., `raster`, `vector`, etc.)
 #' @param cout Logical value, `TRUE` to print a list of commands along with
 #' their descriptions and help URLS to the console (the default).
 #' @param cmd A character string or character vector containing the path to the
 #' algorithm, e.g., `"raster reproject"` or `c("raster", "reproject")`.
 #' Defaults to `"gdal"`, the main entry point to CLI commands.
 #' @param args Either a character vector or a named list containing input
-#' arguments of the algorithm (see section \dQuote{Algorithm argument syntax}).
+#' arguments of the algorithm (see section `Algorithm Argument Syntax` below).
 #' @param parse Logical value, `TRUE` to attempt parsing `args` if they are
 #' given in `gdal_alg()` (the default). Set to `FALSE` to instantiate the
 #' algorithm without parsing arguments. The \code{$parseCommandLineArgs()}
 #' method on the returned object can be called to parse arguments and obtain
 #' the result of that, with potentially useful error messages.
 #'
-#' @section Algorithm argument syntax:
+#' @section Algorithm Argument Syntax:
 #' Arguments are given in \R as a character vector or named list, but
 #' otherwise syntax basically matches the GDAL specification for arguments as
 #' they are given on the command line. Those specifications are listed here
@@ -75,76 +83,77 @@
 #' formats. Programmatic usage also allows passing and receiving datasets as
 #' objects (i.e., `GDALRaster` or `GDALVector`), in addition to dataset names
 #' (e.g., filename, URL, database connection string).
-#'
 #' * Commands accept one or several positional arguments, typically for dataset
-#' names (or, in \R, as `GDALRaster` or `GDALVector` datasets). The order is
+#' names (or in \R as `GDALRaster` or `GDALVector` datasets). The order is
 #' input(s) first, output last. Positional arguments can also be specified as
 #' named arguments, if preferred to avoid any ambiguity.
 #' * Named arguments have:
-#'   * at least one "long" name, preceded by two dash characters when specified
-#'   on the command line,
+#'   * at least one "long" name, preceded by two dash characters
 #'   * optionally, auxiliary long names (i.e., aliases),
 #'   * and optionally a one-letter short name, preceded by a single dash
-#'   character on the command line, e.g., `-f, --of, --format, --output-format
-#'   <OUTPUT-FORMAT>`
-#' * Boolean arguments are specified by just specifying the argument name. In
-#' \R list format, the named element must be assigned a value of logical `TRUE`.
+#'   character, e.g., \code{-f, --of, --format, --output-format <OUTPUT-FORMAT>}
+#' * Boolean arguments are specified by just specifying the argument name in
+#' character vector format. In \R `list` format, the named element must be
+#' assigned a value of logical `TRUE`.
 #' * Arguments that require a value are specified like:
-#'   * `-f VALUE` for one-letter short names.
-#'   * `--format VALUE` or `--format=VALUE` for long names.
-#'   * in a named list, this would look like:\cr
-#'   `arg_list$format <- VALUE`
+#'   * \code{-f VALUE} for one-letter short names
+#'   * \code{--format VALUE} or \code{--format=VALUE} for long names
+#'   * in a named list, this might look like: \code{args$format <- VALUE}
 #' * Some arguments can be multi-valued. Some of them require all values to be
-#' packed together and separated with comma. This is, e.g., the case of:
-#' `--bbox <BBOX> Clipping bounding box as xmin,ymin,xmax,ymax`, e.g.,
-#' `--bbox=2.1,49.1,2.9,49.9`.
+#' packed together and separated with comma. This is, e.g., the case of:\cr
+#' \code{--bbox <BBOX> Clipping bounding box as xmin,ymin,xmax,ymax} \cr
+#' e.g., \code{--bbox=2.1,49.1,2.9,49.9}
 #' * Others accept each value to be preceded by a new mention of the argument
-#' name, e.g., `--co COMPRESS=LZW --co TILED=YES`. For that one, if the value
-#' of the argument does not contain commas, the packed form is also accepted:
-#' `--co COMPRESS=LZW,TILED=YES`. Note that repeated mentions of an argument
-#' are possible in the character vector format for argument input, whereas
-#' arguments given in named list format must used argument long names as the
-#' list element names, and the packed format for the values (which can be a
-#' character vector or numeric vector of values).
+#' name, e.g., \code{c("--co", "COMPRESS=LZW", "--co", "TILED=YES")}. For that
+#' one, if the value of the argument does not contain commas, the packed form
+#' is also accepted: \code{--co COMPRESS=LZW,TILED=YES}. Note that repeated
+#' mentions of an argument are possible in the character vector format for
+#' argument input, whereas arguments given in named list format must use
+#' argument long names as the list element names, and the packed format for the
+#' values (which can be a character vector or numeric vector of values).
 #' * Named arguments can be placed before or after positional arguments.
 #'
-#' @section Development status:
+#' @section Development Status:
 #' The GDAL Command Line Interface Modernization was first introduced in the
 #' [GDAL 3.11.0 release](https://github.com/OSGeo/gdal/releases/tag/v3.11.0)
-#' (2025-05-09). The GDAL project states that the new CLI \dQuote{is
+#' (2025-05-09). The GDAL project provides warning that the new CLI "is
 #' provisionally provided as an alternative interface to GDAL and OGR command
 #' line utilities. The project reserves the right to modify, rename,
 #' reorganize, and change the behavior until it is officially frozen via PSC
-#' vote in a future major GDAL release.... Your usage of it should have no
-#' expectation of compatibility until that time.}
+#' vote in a future major GDAL release. The utility needs time to mature,
+#' benefit from incremental feedback, and explore enhancements without carrying
+#' the burden of full backward compatibility. Your usage of it should have no
+#' expectation of compatibility until that time."
 #' (\url{https://gdal.org/en/latest/programs/#gdal-application})
 #'
-#' Initial bindings to enable programmatic use of the new CLI algorithms from \R
+#' Initial bindings to enable programmatic use of the CLI algorithms from \R
 #' were added in \pkg{gdalraster} 2.2.0, and will evolve over future releases.
-#' The bindings are considered experimental until the upstream API is declared
-#' stable. Breaking changes in minor version releases are possible until then.
+#' *The bindings are considered experimental until the upstream API is declared
+#' stable*. Breaking changes in minor version releases are possible until then.
+#' Please use with those cautions in mind. Bug reports may be filed at:
+#' \url{https://github.com/USDAForestService/gdalraster/issues}.
 #'
 #' @note
 #' Commands do not require the leading `"gdal"` root node. They may begin
 #' with a top-level command (e.g., `"raster"`, `"vector"`, etc.).
 #'
 #' When using argument names as the element names of a list, the underscore
-#' character can be optionally substituted for the dash characters that are
-#' used in some names. This avoids having to surround names in backticks when
-#' they are used to access list elements in the form `args$arg_name` (the form
-#' `args[["arg-name"]]` also works).
+#' character can be substituted for the dash characters that are used in some
+#' names. This avoids having to surround names in backticks when they are used
+#' to access list elements in the form \code{args$arg_name} (the form
+#' \code{args[["arg-name"]]} also works).
 #'
 #' @seealso
 #' [`GDALAlg-class`][GDALAlg]
 #'
-#' \dQuote{gdal} Command Line Interface (CLI):\cr
+#' `gdal` Command Line Interface (CLI) \cr
 #' \url{https://gdal.org/en/stable/programs/index.html}
 #'
-#' Using \dQuote{gdal} CLI algorithms from R:\cr
+#' Using `gdal` CLI algorithms from R\cr
 #' \url{https://usdaforestservice.github.io/gdalraster/articles/use-gdal-cli-from-r.html}
 #'
-#' @examplesIf gdal_version_num() >= gdalraster:::.GDALALG_MIN_GDAL
-#' ## top-level gdal commands
+#' @examplesIf length(gdal_global_reg_names()) > 0
+#' ## top-level commands
 #' gdal_commands(recurse = FALSE)
 #'
 #' ## convert storml_elev.tif to GeoPackage raster
@@ -225,7 +234,7 @@
 #' ds$close()
 #' deleteDataset(f_out)
 #'
-#' ## "pipeline" syntax
+#' ## pipeline syntax
 #' # "raster pipeline" example 2 from:
 #' # https://gdal.org/en/latest/programs/gdal_raster_pipeline.html
 #' # serialize the command to reproject a GTiff file into GDALG format, and
@@ -255,10 +264,9 @@
 #' ds$close()
 #' unlink(f_out)
 #' @export
-gdal_commands <- function(contains = "", recurse = TRUE, cout = TRUE) {
-    if (gdal_version_num() < .GDALALG_MIN_GDAL) {
-        stop("gdal_commands() requires GDAL >= ", .GDALALG_MIN_GDAL_STR,
-             call. = FALSE)
+gdal_commands <- function(contains = NULL, recurse = TRUE, cout = TRUE) {
+    if (gdal_version_num() < gdal_compute_version(3, 11, 3)) {
+        stop("gdal_commands() requires GDAL >= 3.11.3", call. = FALSE)
     }
 
     if (missing(contains) || is.null(contains) || all(is.na(contains)))
@@ -284,9 +292,8 @@ gdal_commands <- function(contains = "", recurse = TRUE, cout = TRUE) {
 #' @name gdal_cli
 #' @export
 gdal_usage <- function(cmd = NULL) {
-    if (gdal_version_num() < .GDALALG_MIN_GDAL) {
-        stop("gdal_usage() requires GDAL >= ", .GDALALG_MIN_GDAL_STR,
-             call. = FALSE)
+    if (gdal_version_num() < gdal_compute_version(3, 11, 3)) {
+        stop("gdal_usage() requires GDAL >= 3.11.3", call. = FALSE)
     }
 
     alg <- gdal_alg(cmd)
@@ -297,9 +304,8 @@ gdal_usage <- function(cmd = NULL) {
 #' @name gdal_cli
 #' @export
 gdal_run <- function(cmd, args) {
-    if (gdal_version_num() < .GDALALG_MIN_GDAL) {
-        stop("gdal_run() requires GDAL >= ", .GDALALG_MIN_GDAL_STR,
-             call. = FALSE)
+    if (gdal_version_num() < gdal_compute_version(3, 11, 3)) {
+        stop("gdal_run() requires GDAL >= 3.11.3", call. = FALSE)
     }
 
     if (missing(cmd) || is.null(cmd) || all(is.na(cmd)))
@@ -332,9 +338,8 @@ gdal_run <- function(cmd, args) {
 #' @name gdal_cli
 #' @export
 gdal_alg <- function(cmd = NULL, args = NULL, parse = TRUE) {
-    if (gdal_version_num() < .GDALALG_MIN_GDAL) {
-        stop("gdal_alg() requires GDAL >= ", .GDALALG_MIN_GDAL_STR,
-             call. = FALSE)
+    if (gdal_version_num() < gdal_compute_version(3, 11, 3)) {
+        stop("gdal_alg() requires GDAL >= 3.11.3", call. = FALSE)
     }
 
     if (missing(cmd) || is.null(cmd) || all(is.na(cmd)))
@@ -371,9 +376,16 @@ gdal_alg <- function(cmd = NULL, args = NULL, parse = TRUE) {
     return(alg)
 }
 
-#' helper function to print usage to the console
-#' called from GDALAlg::usage() in src/gdalalg.cpp
+#' @name gdal_cli
+#' @export
+gdal_global_reg_names <- function() {
+    return(.gdal_global_reg_names())
+}
+
+# helper function to print usage to the console
+# called from GDALAlg::usage() in src/gdalalg.cpp
 #' @noRd
+#' @export
 .print_alg_usage <- function(cmd) {
     alg <- new(GDALAlg, cmd)
     alginfo <- alg$info()
