@@ -411,7 +411,7 @@ warp <- function(src_files,
 #' @note
 #' The returned dataset will have no associated filename for itself. If you
 #' want to write the virtual dataset to a VRT file, use the
-#' `$setFilename()` method on the returned `GDALRaster` object to assign a
+#' \code{$setFilename()} method on the returned `GDALRaster` object to assign a
 #' filename before it is closed.
 #'
 #' @examples
@@ -468,5 +468,5 @@ autoCreateWarpedVRT <- function(src_ds, dst_wkt, resample_alg, src_wkt = "",
     ds <- new(GDALRaster, src_ds, dst_wkt, resample_alg, src_wkt, max_err,
               alpha_band, TRUE, TRUE)
 
-    return(ds);
+    return(ds)
 }
