@@ -8,7 +8,7 @@ knitr::opts_chunk$set(
 library(gdalraster)
 
 tcc_file <- system.file("extdata/storml_tcc.tif", package="gdalraster")
-ds <- new(GDALRaster, tcc_file, read_only=TRUE)
+ds <- new(GDALRaster, tcc_file, read_only = TRUE)
 
 ## -----------------------------------------------------------------------------
 ds
@@ -47,22 +47,22 @@ print(paste("Pixel size:", gt[2], gt[6]))
 
 ## -----------------------------------------------------------------------------
 # block	size
-ds$getBlockSize(band=1)
+ds$getBlockSize(band = 1)
 
 # data type
-ds$getDataTypeName(band=1)
+ds$getDataTypeName(band = 1)
 
 # nodata value
-ds$getNoDataValue(band=1)
+ds$getNoDataValue(band = 1)
 
 # min, max, mean, sd of pixel values in the band
-ds$getStatistics(band=1, approx_ok = FALSE, force = TRUE)
+ds$getStatistics(band = 1, approx_ok = FALSE, force = TRUE)
 
 # does this band have overviews? (aka "pyramids")
-ds$getOverviewCount(band=1)
+ds$getOverviewCount(band = 1)
 
 # does this band have a color table?
-col_tbl <- ds$getColorTable(band=1)
+col_tbl <- ds$getColorTable(band = 1)
 if (!is.null(col_tbl))
   head(col_tbl)
 
@@ -82,7 +82,7 @@ typeof(rowdata)
 head(rowdata)
 
 ## ----fig.width=6, fig.height=4, dev="png"-------------------------------------
-plot_raster(ds, legend=TRUE, main="Storm Lake Tree Canopy Cover (%)")
+plot_raster(ds, legend = TRUE, main = "Storm Lake Tree Canopy Cover (%)")
 
 ## -----------------------------------------------------------------------------
 # close the dataset for proper cleanup
@@ -90,54 +90,51 @@ ds$close()
 
 ## -----------------------------------------------------------------------------
 lcp_file <- system.file("extdata/storm_lake.lcp", package="gdalraster")
-tif_file <- paste0(tempdir(), "/", "storml_lndscp.tif")
-opt <- c("COMPRESS=LZW")
-createCopy(format = "GTiff",
-           dst_filename = tif_file,
-           src_filename = lcp_file,
-           options = opt)
+tif_file <- file.path(tempdir(), "storml_lndscp.tif")
+ds <- createCopy(format = "GTiff",
+                 dst_filename = tif_file,
+                 src_filename = lcp_file,
+                 options = "COMPRESS=LZW",
+                 return_obj = TRUE)
 
-file.size(lcp_file)
-file.size(tif_file)
-
-ds <- new(GDALRaster, tif_file, read_only=FALSE)
-
-# band=0 for dataset-level metadata:
-ds$getMetadata(band=0, domain="IMAGE_STRUCTURE")
+# band = 0 for dataset-level metadata:
+ds$getMetadata(band = 0, domain = "IMAGE_STRUCTURE")
 
 # set nodata value for all bands
 for (band in 1:ds$getRasterCount())
   ds$setNoDataValue(band, -9999)
 
 # band 2 of an LCP file is slope degrees
-ds$getStatistics(band=2, approx_ok=FALSE, force=TRUE)
+ds$getStatistics(band = 2, approx_ok = FALSE, force = TRUE)
 ds$close()
 
+vsi_stat(lcp_file, "size")
+vsi_stat(tif_file, "size")
+
 ## -----------------------------------------------------------------------------
 getCreationOptions("GTiff", "COMPRESS")
 
 getCreationOptions("GTiff", "SPARSE_OK")
 
 ## -----------------------------------------------------------------------------
-new_file <- paste0(tempdir(), "/", "newdata.tif")
-create(format = "GTiff",
-       dst_filename = new_file,
-       xsize = 143,
-       ysize = 107,
-       nbands = 1,
-       dataType = "Int16")
+new_file <- file.path(tempdir(), "newdata.tif")
+ds <- create(format = "GTiff",
+             dst_filename = new_file,
+             xsize = 143,
+             ysize = 107,
+             nbands = 1, 
+             dataType = "Int16",
+             return_obj = TRUE)
 
 ## -----------------------------------------------------------------------------
-ds <- new(GDALRaster, new_file, read_only=FALSE)
-
 # EPSG:26912 - NAD83 / UTM zone 12N
 ds$setProjection(epsg_to_wkt(26912))
 
 gt <- c(323476.1, 30, 0, 5105082.0, 0, -30)
 ds$setGeoTransform(gt)
 
-ds$setNoDataValue(band=1, -9999)
-ds$fillRaster(band=1, -9999, 0)
+ds$setNoDataValue(band = 1, -9999)
+ds$fillRaster(band = 1, -9999, 0)
 
 # ...
 
