@@ -83,8 +83,7 @@ test_that("createCopy writes correct output", {
     createCopy(format="GTiff", dst_filename=tif_file, src_filename=lcp_file,
                options=options)
     ds <- new(GDALRaster, tif_file, read_only=FALSE)
-    files <- ds$getFileList()
-    on.exit(unlink(files))
+    on.exit(deleteDataset(tif_file))
     md <- ds$getMetadata(band=0, domain="IMAGE_STRUCTURE")
     expect_equal(md, c("COMPRESSION=LZW", "INTERLEAVE=PIXEL"))
     for (band in 1:ds$getRasterCount())
@@ -246,13 +245,11 @@ test_that("bandCopyWholeRaster writes correct output", {
     opt <- c("COMPRESSED=YES", "SKIP_HOLES=YES")
     bandCopyWholeRaster(b5_tmp, 1, dst_file, 5, options=opt)
     ds <- new(GDALRaster, b5_tmp, read_only=TRUE)
-    files <- ds$getFileList()
-    on.exit(unlink(files))
+    on.exit(deleteDataset(b5_tmp))
     src_stats <- ds$getStatistics(band=1, approx_ok=FALSE, force=TRUE)
     ds$close()
     ds <- new(GDALRaster, dst_file, read_only=TRUE)
-    files <- ds$getFileList()
-    on.exit(unlink(files))
+    on.exit(deleteDataset(dst_file), add=TRUE)
     dst_stats <- ds$getStatistics(band=5, approx_ok=FALSE, force=TRUE)
     ds$close()
     expect_equal(src_stats, dst_stats)
